@@ -1,6 +1,8 @@
 
 context("Test mc-all build")
 
+skip("")
+
 opts <- antaresRead::setSimulationPath(tail(studyPathS, 1))
 
 outh <- readAntares(areas = "all",
@@ -14,7 +16,7 @@ outm <- readAntares(areas = "all",
 outa <- readAntares(areas = "all",
                     links = "all", timeStep = "annual", showProgress = FALSE)
 
-aggregateResult(opts, timestep = "hourly", writeOutput = TRUE)
+aggregateResult(opts, timestep = "hourly", writeOutput = TRUE, verbose = 0)
 
 outhafter <- readAntares(areas = "all", links = "all",
                          showProgress = FALSE)
